use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( examples lib script t xt ) );
__DATA__
Anwar
Contributors
Dist
Eichner
Etheridge
Fredric
Git
Horsfall
Ioan
Karen
Kent
Klaus
Matthew
Mohammad
Plugin
Ricardo
Rogers
Signes
Zilla
ether
ioan
irc
kentnl
klaus03
lib
mohammad
rjbs
wolfsage
